@@ -204,7 +204,9 @@ static const struct ena_stats ena_stats_ena_com_phc_strings[] = {
 	ENA_STAT_ENA_COM_PHC_ENTRY(phc_cnt),
 	ENA_STAT_ENA_COM_PHC_ENTRY(phc_exp),
 	ENA_STAT_ENA_COM_PHC_ENTRY(phc_skp),
-	ENA_STAT_ENA_COM_PHC_ENTRY(phc_err),
+	ENA_STAT_ENA_COM_PHC_ENTRY(phc_err_dv),
+	ENA_STAT_ENA_COM_PHC_ENTRY(phc_err_ts),
+	ENA_STAT_ENA_COM_PHC_ENTRY(phc_err_eb),
 };
 
 #define ENA_STATS_ARRAY_GLOBAL		ARRAY_SIZE(ena_stats_global_strings)
@@ -490,6 +492,25 @@ static int ena_get_ts_info(struct net_device *netdev,
 
 	info->phc_index = ena_phc_get_index(adapter);
 
+	if (ena_com_hw_timestamping_supported(adapter->ena_dev)) {
+		if (adapter->hw_ts_state.hw_tx_supported !=
+		    ENA_ADMIN_HW_TIMESTAMP_TX_SUPPORT_NONE) {
+			info->so_timestamping |= SOF_TIMESTAMPING_TX_HARDWARE |
+						 SOF_TIMESTAMPING_RAW_HARDWARE;
+
+			info->tx_types = BIT(HWTSTAMP_TX_OFF) |
+					 BIT(HWTSTAMP_TX_ON);
+		}
+
+		if (adapter->hw_ts_state.hw_rx_supported !=
+		    ENA_ADMIN_HW_TIMESTAMP_RX_SUPPORT_NONE) {
+			info->so_timestamping |= SOF_TIMESTAMPING_RX_HARDWARE;
+
+			info->rx_filters = BIT(HWTSTAMP_FILTER_NONE) |
+					   BIT(HWTSTAMP_FILTER_ALL);
+		}
+	}
+
 	return 0;
 }
 
